"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_line_commands_1 = __importDefault(require("command-line-commands"));
const convertOptionDefinition_1 = require("./convertOptionDefinition");
const helpOption_1 = require("./commands/helpOption");
const command_line_args_1 = __importDefault(require("command-line-args"));
const runCliRecursive = async function ({ command, argv, showUsage, recommendCommand, level, ancestorOptions, ancestorNames }) {
    const commandPath = [...ancestorNames, command.name];
    const optionDefinitions = command.optionDefinitions.
        map((optionDefinition) => convertOptionDefinition_1.convertOptionDefinition({ optionDefinition }));
    const optionDefinitionsWithHelp = [
        ...optionDefinitions,
        helpOption_1.helpOption
    ];
    const { _unknown, ...options } = command_line_args_1.default(optionDefinitionsWithHelp, { argv, stopAtFirstUnknown: true });
    if (options.help) {
        /* eslint-disable no-console */
        console.log(showUsage({ commandPath }));
        /* eslint-enable no-console */
        return;
    }
    const mergedOptions = {
        ...ancestorOptions,
        ...options
    };
    if (_unknown === undefined || command.ignoreUnknownOptions) {
        try {
            return await command.handle({
                options: mergedOptions,
                showUsage,
                level,
                ancestors: ancestorNames
            });
        }
        catch (ex) {
            /* eslint-disable no-console */
            console.log(ex);
            /* eslint-enable no-console */
            /* eslint-disable unicorn/no-process-exit */
            return process.exit(1);
            /* eslint-enable unicorn/no-process-exit */
        }
    }
    if (command.subcommands === undefined || Object.keys(command.subcommands).length === 0) {
        const unknowOption = _unknown[0];
        /* eslint-disable no-console */
        console.log(`Unknown option '${unknowOption}'.`);
        /* eslint-enable no-console */
        /* eslint-disable unicorn/no-process-exit */
        return process.exit(1);
        /* eslint-enable unicorn/no-process-exit */
    }
    try {
        const { command: subCommandName, argv: subArgv } = command_line_commands_1.default(Object.keys(command.subcommands), 
        // Pass a copy of the _unknown array, since the commandLineCommands
        // function mutates the parameter in-place. To avoid this, spread it and
        // wrap the values within a new array.
        [..._unknown]);
        const subCommand = command.subcommands[subCommandName];
        return await runCliRecursive({
            command: subCommand,
            argv: subArgv,
            showUsage,
            recommendCommand,
            level: level + 1,
            ancestorOptions: mergedOptions,
            ancestorNames: commandPath
        });
    }
    catch {
        const unknownCommand = _unknown[0];
        const recommendedCommand = recommendCommand({ commandPath: [...commandPath, unknownCommand] });
        /* eslint-disable no-console */
        console.log(`Unknown command '${unknownCommand}'. Did you mean '${recommendedCommand}'?`);
        /* eslint-enable no-console */
        /* eslint-disable unicorn/no-process-exit */
        process.exit(1);
        /* eslint-enable unicorn/no-process-exit */
    }
};
exports.runCliRecursive = runCliRecursive;
